//------------------------------------------------------------------------------
// CERN BE-CO-HT
// Mock Turtle
// https://gitlab.cern.ch/coht/mockturtle
//------------------------------------------------------------------------------
//
// unit name:   MTConfigRom
//
// description: A ROM to hold the various configuration values of the current
// MockTurtle implementation.
//
//------------------------------------------------------------------------------
// Copyright CERN 2018
//------------------------------------------------------------------------------
// Copyright and related rights are licensed under the Solderpad Hardware
// License, Version 2.0 (the "License"); you may not use this file except
// in compliance with the License. You may obtain a copy of the License at
// http://solderpad.org/licenses/SHL-2.0.
// Unless required by applicable law or agreed to in writing, software,
// hardware and materials distributed under this License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions
// and limitations under the License.
//------------------------------------------------------------------------------

`ifndef __MOCK_TURTLE_CONFIG_ROM_INCLUDED
 `define __MOCK_TURTLE_CONFIG_ROM_INCLUDED

 `include "simdrv_defs.svh"

class MTConfigRom;
   protected CBusAccessor m_acc;
   protected uint64_t     m_base;
   protected uint32_t     m_size;
   protected uint32_t     m_data [];

   function new ( CBusAccessor acc, uint64_t base, uint64_t size = 512);
      m_acc  = acc;
      m_base = base;
      m_size = size;
      m_data = new[m_size];
   endfunction // new

   task _readl (  uint32_t r, ref uint32_t v );
      uint64_t tmp;
      m_acc.read ( m_base + r, tmp );
      v= tmp;
   endtask // _readl

   task init ( );
      int i;

      for ( i = 0; i < m_size; i++ )
	begin
	   _readl ( 4*i, m_data[i] );
	end
   endtask // init

   function uint32_t read ( uint32_t pos );
      return m_data[pos];
   endfunction // read

   function uint32_t getAppID();
      return read(5);
   endfunction // getAppID

   function uint32_t getCoreCount();
      return read(6);
   endfunction // getCoreCount

   function uint32_t getHmqSlotCount(int core);
      return read(16 + core);
   endfunction // getHmqSlotCount

   function uint32_t getRmqSlotCount(int core);
      return read(24 + core);
   endfunction // getRmqSlotCount

   task dump ();
      int i;

      for ( i = 0; i < m_size; i++ )
	begin
	   $display ("%4d: %8x", i, read(i));
	end
   endtask // dump

   task pretty_print ( );
      int i, j;
      int cpu_count = getCoreCount();

      $display ( "Configuration ROM contents:" );
      $display ( "  Magic Value     : %0s", read(0) );
      $display ( "  MT revision     : %0d", read(1) );
      $display ( "  Clock Frequency : %0d", read(3) );
      $display ( "  WR support      : %0s", read(4) & 1 ? "yes" : "no" );
      $display ( "  Application ID  : %0x", getAppID() );
      $display ( "  Shared mem size : %0d", read(7) );
      $display ( "  CPU count       : %0d", cpu_count );

      for ( i = 0; i < cpu_count; i++ )
	begin
	   int hmq_slots = getHmqSlotCount(i);
	   int rmq_slots = getRmqSlotCount(i);
	   $display ( "    CPU #%0d", i );
	   $display ( "      Local mem size : %0d", read(8+i) );
	   $display ( "      HMQ slots      : %0d", hmq_slots );
	   for ( j = 0; j < hmq_slots; j++ )
	     begin
		uint32_t val;
		$display ( "        HMQ #%0d", j );
		val = read(128 + 16 * i + 2 * j);
		$display ( "          Entries bits : %0d", (val & 'h00ff0000) >> 16 );
		$display ( "          Width bits   : %0d", (val & 'h0000ff00) >> 8 );
		$display ( "          Header bits  : %0d", (val & 'h000000ff) );
		val = read(128 + 16 * i + 2 * j + 1);
		$display ( "          Endpoint ID  : %0x", val );
	     end
	   $display ( "      RMQ slots      : %0d", rmq_slots );
	   for ( j = 0; j < hmq_slots; j++ )
	     begin
		uint32_t val;
		$display ( "        RMQ #%0d", j );
		val = read(256 + 16 * i + 2 * j);
		$display ( "          Entries bits : %0d", (val & 'h00ff0000) >> 16 );
		$display ( "          Width bits   : %0d", (val & 'h0000ff00) >> 8 );
		$display ( "          Header bits  : %0d", (val & 'h000000ff) );
		val = read(256 + 16 * i + 2 * j + 1);
		$display ( "          Endpoint ID  : %0x", val );
	     end
	end

   endtask // pretty_print

endclass // MockTurtleConfigRom

`endif // `ifndef __MOCK_TURTLE_CONFIG_ROM_DRIVER_INCLUDED
